import sys
import os
import subprocess

from sys import platform
from importlib import resources

from ansurr.functions import check_quiet_print

def calc_secondary_structure(pdb,output_dir='',quiet=False):

    if platform == "linux" or platform == "linux2":
        with resources.path("ansurr.bin", "dssp.bin") as f:  # use this for the package
        #with resources.path("bin", "dssp.bin") as f: 
            dssp_file_path = f

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    out = open(output_dir+os.path.basename(os.path.splitext(pdb)[0])+".dssp","w")

    dssp_output = subprocess.run([dssp_file_path, pdb],stdout=subprocess.PIPE, universal_newlines=True, stderr=open(os.devnull, 'w'))

    start = 0
    for line in dssp_output.stdout.split('\n'):
        if start == 1:
            try:
                resi = str(int(line[5:10]))
                resn = line[13]
                ss = line[16]
                if ss == ' ':
                    ss = '.'
                out.write(resi+' '+resn+' '+ss+'\n')
            except:
                pass

        elif "#  RESIDUE AA" in line:
            start = 1
    out.close()

def main():

    calc_secondary_structure(sys.argv[1])
    sys.exit(0)
   
if __name__ == "__main__":
    main()


import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import sys
import numpy as np

from ansurr import adjust_text as at

def plot_scores(output_dir='',cyrange=False):
	data = {}
	for line in open(output_dir+'scores.out','r'):
		if line[:4] == 'PDB:':
			line = line.split()
			pdb = line[1][:[pos for pos, char in enumerate(line[1]) if char == '_'][-1]] # accounts for "_" appearing in structure name
			model = int(line[1].split('_')[-1])
			shifts = line[3]
			shift_perc = int(line[5])
			if line[9] != 'nan':
				if pdb not in data:
					if cyrange:
						data[pdb] = {'model':[model],'rmsd':[float(line[13])],'corr':[float(line[9])],'rmsd_welldefined':[float(line[21])],'corr_welldefined':[float(line[17])],'shifts':shifts,'shift_perc':[shift_perc]}
					else:
						data[pdb] = {'model':[model],'rmsd':[float(line[13])],'corr':[float(line[9])],'shifts':shifts,'shift_perc':[shift_perc]}

				else:
					data[pdb]['model'].append(model)
					data[pdb]['rmsd'].append(float(line[13]))
					data[pdb]['corr'].append(float(line[9]))
					data[pdb]['shift_perc'].append(shift_perc)
					if cyrange:
						data[pdb]['rmsd_welldefined'].append(float(line[21]))
						data[pdb]['corr_welldefined'].append(float(line[17]))

				
	for d in enumerate(data):
		plt.figure(d[0])
		plt.scatter(data[d[1]]['rmsd'], data[d[1]]['corr'], c='black',s=8)

		labels = [plt.text(data[d[1]]['rmsd'][i], data[d[1]]['corr'][i], data[d[1]]['model'][i],
						   ha='center', va='center', color='blue',size=6) for i in range(len(data[d[1]]['rmsd']))]

		plt.ylabel('correlation score',size=12)
		plt.xlabel('RMSD score',size=12)
		plt.axis('scaled')  
		plt.xlim(-5,105)
		plt.ylim(-5,105)
		shift_perc = int(round(np.nanmean(data[d[1]]['shift_perc']),0)) # just in case models don't have the same shift completeness - very unlikely
		if shift_perc < 75:
			shift_perc_out = str(shift_perc) + ' (unreliable!)'
		else:
			shift_perc_out = str(shift_perc)

		plt.title('Structure: '+d[1]+'\nShifts: '+data[d[1]]['shifts']+' Shift%: '+shift_perc_out)
		at.adjust_text(labels,expand_text=(1.75, 1.75), expand_points=(1.75, 1.75),arrowprops=dict(arrowstyle='->', color='red', alpha=0.6, linewidth=0.5))
		plt.tight_layout()
		plt.savefig(output_dir+d[1]+'_'+data[d[1]]['shifts']+'.png',dpi=300)


		if cyrange:
			plt.cla()
			plt.scatter(data[d[1]]['rmsd_welldefined'], data[d[1]]['corr_welldefined'], c='black',s=8)
			labels_cyrange = [plt.text(data[d[1]]['rmsd_welldefined'][i], data[d[1]]['corr_welldefined'][i], data[d[1]]['model'][i],
						   ha='center', va='center', color='blue',size=6) for i in range(len(data[d[1]]['rmsd_welldefined']))]

			plt.ylabel('correlation score',size=12)
			plt.xlabel('RMSD score',size=12)
			plt.axis('scaled')  
			plt.xlim(-5,105)
			plt.ylim(-5,105)
			shift_perc = int(round(np.nanmean(data[d[1]]['shift_perc']),0)) # have sep shift completness for well defined region?
			if shift_perc < 75:
				shift_perc_out = str(shift_perc) + ' (unreliable!)'
			else:
				shift_perc_out = str(shift_perc)

			plt.title('Structure: '+d[1]+'\n(WD) Shifts: '+data[d[1]]['shifts']+' Shift%: '+shift_perc_out)
			at.adjust_text(labels_cyrange,expand_text=(1.75, 1.75), expand_points=(1.75, 1.75),arrowprops=dict(arrowstyle='->', color='red', alpha=0.6, linewidth=0.5))
			plt.tight_layout()
			plt.savefig(output_dir+d[1]+'_'+data[d[1]]['shifts']+'_welldefined.png',dpi=300)

def main():

    plot_scores(sys.argv[1])
    sys.exit(0)
   
if __name__ == "__main__":
    main()